package com.zoho.scrapy;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.sql.*;
import java.util.HashSet;

/**
 * @author sumanth-13934
 */
public class PopulateDBSync {
    private static FileReader reader;
    private static int batchSize = 1000;
    private static long count = 0;

    public PopulateDBSync(String fileName) throws FileNotFoundException {
        reader = new FileReader(fileName);
    }

    public void populate(){
        try{
            DBConnection dbConnection = new DBConnection();
            Connection conn = dbConnection.getDBConnection();
            DatabaseMetaData databaseMetaData = conn.getMetaData();;
            ResultSet resultSet = databaseMetaData.getTables(null,null,null, new String[] {"TABLE"});
            HashSet<String> tables = new HashSet<>();
            while (resultSet.next()){
                tables.add(resultSet.getString("TABLE_NAME"));
            }

            //Creating Table to populate company names in table.
            if(!tables.contains("lcmpydb")){
                Statement createTableSmt = conn.createStatement();
                String createLCmpyDb = "CREATE TABLE lcmpydb(" +
                        "id BIGSERIAL PRIMARY KEY,"+
                        "cmpy TEXT UNIQUE,"+
                        "statuscode SMALLINT,"+
                        "started BIGINT,"+
                        "ended BIGINT,"+
                        "retry SMALLINT,"+
                        "status SMALLINT"+
                        ");";
                if(createTableSmt.executeUpdate(createLCmpyDb) == 0){
                    System.out.println("created lcmpydb table...");
                }
                else {
                    System.out.println("Error occurred while creating lcmpydb table...");
                }
            }
            else {
                System.out.println("lcmpydb table already existed...");
            }

            if(!tables.contains("similarcmpy")){
                Statement createTableSmt = conn.createStatement();
                String createLCmpyDb = "CREATE TABLE similarcmpy(" +
                        "id BIGSERIAL PRIMARY KEY,"+
                        "cmpy TEXT UNIQUE"+

                        ");";
                if(createTableSmt.executeUpdate(createLCmpyDb) == 0){
                    System.out.println("created similarcmpy table...");
                }
                else {
                    System.out.println("Error occurred while creating similarcmpy table...");
                }
            }
            else {
                System.out.println("similarcmpy table already existed...");
            }

            //populating into table.
            if(Config.forceInsertInToDB){
                System.out.println("Inserting to db...");
                long start = System.currentTimeMillis();
                BufferedReader bufferedReader = new BufferedReader(reader);
                String line;
                String insertQuery = "insert into lcmpydb values(DEFAULT,?,?,?,?,?,?) on conflict(cmpy) do nothing;";
                PreparedStatement insertPSmt = conn.prepareStatement(insertQuery);
                while((line=bufferedReader.readLine()) != null){
                    count += 1;
                    insertPSmt.setString(1,line);
                    insertPSmt.setInt(2,0);
                    insertPSmt.setInt(3,0);
                    insertPSmt.setInt(4,0);
                    insertPSmt.setInt(5,0);
                    insertPSmt.setInt(6,2);
                    insertPSmt.addBatch();
                    if(count % batchSize == 0){
                        insertPSmt.executeBatch();
                    }
                }
                if(count % batchSize != 0){
                    insertPSmt.executeBatch();
                }
                insertPSmt.close();
                long end = System.currentTimeMillis();
                double total = (end-start)/1000.0;
                System.out.println("Inserted all records in "+total+" sec.");
            }
            else {
                System.out.println("Force insert into table is Disabled,no data is inserted newly... !");
            }
            conn.close();
        }
        catch (Exception e){
            e.printStackTrace();
        }
    }

}
