package com.zoho.scrapy;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.concurrent.TimeUnit;

/**
 * @author sumanth-13934
 */
public class Reports {
    public static void main(String[] args) throws SQLException, ClassNotFoundException, IOException {
        DBConnection dbConnection = new DBConnection();
        Connection conn = dbConnection.getDBConnection();

        Path path = Paths.get(Config.dataSavePath+"batch"+Config.batchNo+"/reports/");
        Files.createDirectories(path);

        BufferedWriter statWriter = new BufferedWriter(new FileWriter(Config.dataSavePath+"batch"+Config.batchNo+"/reports/stats.txt",true));
        String line;

        String successQuery = "copy (select * from lcmpydb where statuscode=200) to '"+Config.dataSavePath+"/batch"+Config.batchNo+"/reports/success.csv' with (format csv, header);";
        Statement successSmt = conn.createStatement();
        successSmt.execute(successQuery);
        successSmt.close();

        String failsQuery = "copy (select * from lcmpydb where statuscode!=200) to '"+Config.dataSavePath+"/batch"+Config.batchNo+"/reports/fail.csv' with (format csv, header);";
        Statement failSmt = conn.createStatement();
        failSmt.execute(failsQuery);
        failSmt.close();

        String notFoundQuery = "copy (select cmpy from lcmpydb where statuscode=404) to '"+Config.dataSavePath+"/batch"+Config.batchNo+"/reports/404.txt' with CSV;";
        Statement notFoundSmt = conn.createStatement();
        notFoundSmt.execute(notFoundQuery);
        notFoundSmt.close();

        String blockedQuery = "copy (select cmpy from lcmpydb where statuscode!=404 and statuscode!=200) to '"+Config.dataSavePath+"/batch"+Config.batchNo+"/reports/block.txt' with CSV;";
        Statement blockedSmt = conn.createStatement();
        blockedSmt.execute(blockedQuery);
        blockedSmt.close();

        String successCountQuery = "select count(*) from lcmpydb where statuscode=200;";
        Statement successCountSmt = conn.createStatement();
        ResultSet successCountRs = successCountSmt.executeQuery(successCountQuery);
        successCountRs.next();
        long successCount = successCountRs.getLong("count");
        line = "Success Count :: "+successCount;
        statWriter.write(line);
        statWriter.newLine();
        System.out.println(line);
        successCountRs.close();
        successCountSmt.close();

        String blockCountQuery = "select count(*) from lcmpydb where statuscode!=200 and statuscode!=404 and statuscode!=0;";
        Statement blockCountSmt = conn.createStatement();
        ResultSet blockCountRs = blockCountSmt.executeQuery(blockCountQuery);
        blockCountRs.next();
        long blockCount = blockCountRs.getLong("count");
        line = "Blocked Count :: "+blockCount;
        statWriter.write(line);
        statWriter.newLine();
        System.out.println(line);
        blockCountRs.close();
        blockCountSmt.close();

        String notFoundCountQuery = "select count(*) from lcmpydb where statuscode=404;";
        Statement notFoundCountSmt = conn.createStatement();
        ResultSet notFoundCountRs = notFoundCountSmt.executeQuery(notFoundCountQuery);
        notFoundCountRs.next();
        long notFoundCount = notFoundCountRs.getLong("count");
        line = "NotFound Count :: " + notFoundCount;
        statWriter.write(line);
        statWriter.newLine();
        System.out.println(line);
        notFoundCountRs.close();
        notFoundSmt.close();

        String totalQuery = "select count(*) from lcmpydb;";
        Statement totSmt = conn.createStatement();
        ResultSet totRs = totSmt.executeQuery(totalQuery);
        totRs.next();
        double totalUrls = totRs.getDouble("count");
        totRs.close();
        totSmt.close();

        double successRate = (successCount/totalUrls)*100;
        line = "Success Rate  :: " + successRate + " %";
        statWriter.write(line);
        statWriter.newLine();
        System.out.println(line);


        String startTime = "select min(started) from lcmpydb where started!=0;";
        Statement startSmt = conn.createStatement();
        ResultSet startRs = startSmt.executeQuery(startTime);
        startRs.next();
        long start = startRs.getLong("min");
        startRs.close();
        startSmt.close();

        String endTime = "select max(ended) from lcmpydb;";
        Statement endSmt = conn.createStatement();
        ResultSet endRs = endSmt.executeQuery(endTime);
        endRs.next();
        long end = endRs.getLong("max");
        endRs.close();
        endSmt.close();

        String time = estimate(end-start);
        line = "Time taken :: " + time;
        statWriter.write(line);
        statWriter.newLine();
        System.out.println(line);
        statWriter.close();

    }

    private static String estimate(long milliseconds){
        long seconds = TimeUnit.MILLISECONDS.toSeconds(milliseconds);
        long minute = TimeUnit.MILLISECONDS.toMinutes(milliseconds);
        long hours = TimeUnit.MILLISECONDS.toHours(milliseconds);
        long days = TimeUnit.MILLISECONDS.toDays(milliseconds);
        if(milliseconds < 1000){
            return (days +"d:" +hours+"h:" +minute+"m:" +seconds +"s:" +milliseconds +"ms");
        }
        else{
            return (days +"d:" +hours % 24 +"h:" +minute % 60 +"m:" +seconds % 60 +"s");
        }
    }
}
